from typing import Literal


AuthenticationMethodReferenceLiteral = Literal[
    'face',
    'fpt',
    'geo',
    'hwk',
    'iris',
    'kba',
    'mca',
    'mfa',
    'otp',
    'pin',
    'pop',
    'pwd',
    'rba',
    'retina',
    'sc',
    'sms',
    'swk',
    'tel',
    'user',
    'vbm',
    'wia',
]